package ch.usi.inf.sa4.sanmarinoes.smarthut.controller;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.mockito.ArgumentMatchers.anyIterable;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

import ch.usi.inf.sa4.sanmarinoes.smarthut.error.NotFoundException;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.*;
import ch.usi.inf.sa4.sanmarinoes.smarthut.service.DeviceService;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.security.test.context.support.WithMockUser;

@ExtendWith(MockitoExtension.class)
@WithMockUser(username = "user")
public class InputDeviceConnectionControllerTests {

    @Mock private DeviceService deviceService;

    @Mock private KnobDimmerRepository inputRepository;

    @Mock private DimmableRepository<Dimmable> outputRepository;

    private KnobDimmerController knobDimmerController;

    @Mock private Principal mockPrincipal;

    @BeforeEach
    public void setup() {
        when(mockPrincipal.getName()).thenReturn("user");
    }

    @Test
    public void testConnection() throws NotFoundException {
        KnobDimmer knobDimmer = new KnobDimmer();
        DimmableLight dimmableLight = new DimmableLight();
        knobDimmer.addDimmable(dimmableLight);

        when(inputRepository.findByIdAndUsername(anyLong(), anyString()))
                .thenReturn(java.util.Optional.of(knobDimmer));

        when(outputRepository.findByIdAndUsername(anyLong(), anyString()))
                .thenReturn(java.util.Optional.of(dimmableLight));

        when(deviceService.saveAllAsOwner(anyIterable(), anyString()))
                .thenReturn(List.of(new DimmableLight()));

        List<Long> l = new ArrayList<>();
        l.add(10L);

        knobDimmerController =
                new KnobDimmerController(inputRepository, outputRepository, deviceService);

        assertDoesNotThrow(() -> knobDimmerController.addLight(1L, l, mockPrincipal));
    }
}
