package ch.usi.inf.sa4.sanmarinoes.smarthut.models;

import static org.junit.jupiter.api.Assertions.*;

import java.util.Date;
import nl.jqno.equalsverifier.EqualsVerifier;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

@DisplayName("ConfirmationToken tests")
public class ConfirmationTokenTests {
    private ConfirmationToken token;

    @BeforeEach
    private void createToke() {
        User user = new User();
        user.setName("Tizio Sempronio");
        user.setId(42l);
        user.setUsername("xXCoolUserNameXx");
        user.setEmail("realMail@service.ext");
        user.setPassword("alpaca");
        this.token = new ConfirmationToken(user);
    }

    @Test
    @DisplayName("test setId")
    public void testSetId() {
        this.token.setId(34l);
        assertEquals(34l, token.getId());
    }

    @Test
    @DisplayName("test getUSerId")
    public void testGetUserId() {
        assertEquals(42l, token.getUser().getId());
    }

    @Test
    @DisplayName("test setResetPassword")
    public void testSetResetPassword() {
        assertFalse(token.isResetPassword());
        token.setResetPassword(true);
        assertTrue(token.isResetPassword());
    }

    @Test
    @DisplayName("test setTimeStamp")
    public void testSetTimeStamp() {
        Date date = new Date();
        date.setTime(86400000L);
        token.setCreatedDate(date);
        assertEquals(date, token.getCreatedDate());
    }

    @Test
    @DisplayName("test setConfirmToken")
    public void testSetConfirmTOken() {
        assertNotEquals(null, token.getConfirmToken());
        String newToken = "newConfirmationToken";
        token.setConfirmToken(newToken);
        assertEquals(newToken, token.getConfirmToken());
    }

    @Test
    @DisplayName("test setConfirmToken")
    public void equals() {
        User user = new User();
        user.setName("Tizio Sempronio");
        user.setId(42L);
        user.setUsername("xXCoolUserNameXx");
        user.setEmail("realMail@service.ext");
        user.setPassword("alpaca");
        ConfirmationToken t = new ConfirmationToken();
        t.setUser(user);

        assertNotEquals(t, token);
    }

    @Test
    public void testEqualsHashCode() {
        final User u = new User();
        u.setId(1L);
        final User t = new User();
        t.setId(2L);
        EqualsVerifier.forClass(ConfirmationToken.class)
                .withNonnullFields("createdDate")
                .withNonnullFields("confirmToken")
                .withPrefabValues(User.class, u, t)
                .verify();
        assertDoesNotThrow(() -> new ConfirmationToken().hashCode());
    }
}
