package ch.usi.inf.sa4.sanmarinoes.smarthut.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

import ch.usi.inf.sa4.sanmarinoes.smarthut.models.Thermostat;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.ThermostatRepository;
import ch.usi.inf.sa4.sanmarinoes.smarthut.models.User;
import ch.usi.inf.sa4.sanmarinoes.smarthut.socket.SensorSocketEndpoint;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
public class ThermostatServiceTests {

    @InjectMocks private ThermostatService thermostatService;

    @Mock private ThermostatRepository thermostatRepository;

    @Mock private DeviceService deviceService;

    @Mock private ThermostatPopulationService thermostatPopulationService;

    @Mock private SensorSocketEndpoint endpoint;

    @Test
    public void testFakeUpdateAll() {
        when(deviceService.saveAsOwner(any(), any())).thenAnswer(i -> i.getArgument(0));
        when(thermostatRepository.findUser(1L)).thenReturn(new User());

        Thermostat t = new Thermostat();
        t.setOn(true);
        t.setId(1L);
        t.setTargetTemperature(BigDecimal.valueOf(17.0));

        when(thermostatRepository.findAll()).thenReturn(List.of(t));
        doAnswer(
                        i -> {
                            ((Thermostat) i.getArgument(0))
                                    .setMeasuredTemperature(BigDecimal.valueOf(18.0));
                            return null;
                        })
                .when(thermostatPopulationService)
                .populateMeasuredTemperature(t);

        doNothing().when(endpoint).queueDeviceUpdate(any(), any(), eq(false), eq(null), eq(false));

        Assertions.assertDoesNotThrow(() -> thermostatService.fakeUpdateAll());
        assertThat(t.getMode()).isNotEqualTo(Thermostat.Mode.OFF);
    }

    @Test
    public void testFindAll() {
        final Thermostat t = new Thermostat();
        doNothing().when(thermostatPopulationService).populateMeasuredTemperature(t);
        when(thermostatRepository.findAllByUsername("user")).thenReturn(List.of(t));
        assertThat(thermostatService.findAll("user")).containsExactly(t);
    }

    @Test
    public void testFindById() {
        final Thermostat t = new Thermostat();
        when(thermostatRepository.findByIdAndUsername(2L, "user")).thenReturn(Optional.empty());
        when(thermostatRepository.findByIdAndUsername(1L, "user")).thenReturn(Optional.of(t));
        doNothing().when(thermostatPopulationService).populateMeasuredTemperature(t);
        assertThat(thermostatService.findById(2L, "user")).isEmpty();
        assertThat(thermostatService.findById(1L, "user")).isNotEmpty();
    }
}
